/* File hacked from Sunflower demo by @eigenbom. Original copyright below */

// Copyright (c) 2012, the Dart project authors.  Please see the AUTHORS file
// for details. All rights reserved. Use of this source code is governed by a
// BSD-style license that can be found in the LICENSE file.

library sunflower;

import 'dart:html';
import 'dart:math';
import 'dart:async';

const String ORANGE = "orange";
const double SEED_RADIUS = 1.0; // 2;
const int SCALE_FACTOR = 3; // 4;
const num TAU = PI * 2;
const int MAX_D = 600;
const num centerX = MAX_D / 2;
const num centerY = centerX;

final InputElement sliderTwist = querySelector("#sliderTwist");
final InputElement slider = querySelector("#slider");
final InputElement slider2 = querySelector("#slider2");
final InputElement slider3 = querySelector("#slider3");
final InputElement sliderPulse = querySelector("#sliderPulse");
final num PHI = (sqrt(5) + 1) / 2;
int seeds = 0;
double foo = 0.0;
double bar = 0.0;
var start = null;
final CanvasRenderingContext2D context =
  (querySelector("#canvas") as CanvasElement).context2D;

void main() {  
  window.animationFrame.then(step);
}

void step(timestamp){
  var progress;
  if (start == null) start = timestamp;
  progress = timestamp - start;
  
  bar = progress / 1000.0;
  
  draw();
  window.animationFrame.then(step);
}

/// Draw the complete figure for the current number of seeds.
void draw() {
  double twist = double.parse(sliderTwist.value)/800.0;
  seeds = int.parse(slider.value);
  foo = (double.parse(slider2.value)-50.0)/50000.0;  
  context.clearRect(0, 0, MAX_D, MAX_D);
  var s3 = 2*(double.parse(slider3.value)-50.0)/50.0;
  
  var pulse = double.parse(sliderPulse.value)/100.0;
  
  for (var i = 0; i < seeds; i++) {
    final num theta = (twist + foo+s3*1.0*sin(bar*0.0001)) * i * TAU / PHI;
    
    if (cos(theta)>0.01 || sin(theta)>0.01) continue;
    
    final num r = (1 + 0.1*sin(bar)*sin(bar+0.01*i)) * sqrt(i) * (1 + 0.1*sin(i*0.01)) * SCALE_FACTOR;
 
    final double dx = r * cos(theta);
    final double dy = r * sin(theta);
    
    final double rad = (1 + 0.5*sin(theta*4)*cos(r*0.1));
    final double radius = SEED_RADIUS*rad;
    
    final double h = (1 + 0.2*sin(r/10.0 + bar*5*pulse)).clamp(0.0, 1.0)*360;
    final double a = (1 + 0.5*cos(theta))*0.3*rad;
    final String col = "hsla(" + h.toString() + ",100%,50%," + a.toString() + ")"; 
    
    drawSeed(centerX + dx, centerY - dy, col, radius);    
    drawSeed(centerX - dx, centerY - dy, col, radius);
    drawSeed(centerX + dx, centerY + dy, col, radius);    
    drawSeed(centerX - dx, centerY + dy, col, radius);
  }
}

/// Draw a small circle representing a seed centered at (x,y).
void drawSeed(num x, num y, colour, radius) {  
  context..beginPath()
         ..lineWidth = 0
         ..fillStyle = colour
         // ..rect(x, y, 2, 2)    
         ..arc(x, y, radius*SCALE_FACTOR, 0, TAU, false)
         ..fill();
}
